#ifndef API_FMESSAGE_H
#define API_FMESSAGE_H

/*******************************************************
 * TITLE: Message.h
 * AUTHOR: Richard Whittaker
 * CREATION DATE: 1/6/03
 * PURPOSE:
 * This class contains a message string that may need
 * additional formatting.  Formatting is done by passing
 * paramaters into the Message with the parm() method.
 * This class may be used dirctly as a (const char *)
 *******************************************************/

#ifdef WIN32

#define UNSIGNED_LONG_FORMAT "%I64u"
#define SIGNED_LONG_FORMAT "%I64d"

#else

#define UNSIGNED_LONG_FORMAT "%llu"
#define SIGNED_LONG_FORMAT "%lld"

#ifndef __int64
#define __int64 long long
#endif

#endif

#define	FMESSAGE_SIZE		4096
#define FMESSAGE_CONV_SIZE	8192

class API_FMessage
{
public:
	API_FMessage();
	API_FMessage(const char * str);
	API_FMessage(API_FMessage&);
	~API_FMessage();

	// Overloaded operator^ methods.  Need one for each type.
	// I dub this the "Parameter Insertion Operator"

	API_FMessage & operator^(unsigned int arg);
	API_FMessage & operator^(int arg);
	API_FMessage & operator^(char * arg);
	API_FMessage & operator^(const char * arg);
	API_FMessage & operator^(char arg);
	API_FMessage & operator^(float arg);
	API_FMessage & operator^(double arg);
	API_FMessage & operator^(unsigned __int64 arg);
	API_FMessage & operator^(__int64 arg);

	// operator, in case someone wants to append something
	// to the end of the messages (like a \n)
	API_FMessage & operator+(const char * arg);

	// Assignment operator
	API_FMessage& operator=(API_FMessage&) ;

	// (const char *) cast operator
	operator const char *();

private:
	// format is used to replace positional placeholders
	// with the converted parm data
	int format(const char * arg);

	// The actual message
	char *msg;
	char *conv_msg;

	// Tracks the parameter position in the message
	int pos;

};



#endif


