//  Copyright 2001-2002, NeoCore Inc.
//  All Rights Reserved

#include "API_C_dll.h"
#ifndef XMLUTILS_H
#define XMLUTILS_H

#include "NeoException.h"


/**
 * Used to extract data out of server responses.
 */
#define SERVER_FORMAT_COPY		"CopyNumber"
#define SERVER_FORMAT_COUNT 	"Count"
#define SERVER_FORMAT_DELETE    "Deleted-Nodes"
#define SERVER_FORMAT_ERROR		"Error"
#define SERVER_FORMAT_INSERT	"Insert-Nodes"
#define SERVER_FORMAT_MODIFY	"Modified-Nodes"
#define SERVER_FORMAT_MODIFY_MATCHES	"Matching-Nodes"
#define SERVER_FORMAT_SID		"sid"



/** XMLUtils provides basic XML parsing functionality for returned server API method results 
 * @see NeoConnection
 * @see SessionManagedNeoConnection
 *
 */

class HTTPCLIENT4_API XMLUtils {
public:
	static char* getSID(char* XML);

	/**
	* @exception  NullPointerException - If the XML did not contain the copy tag
	* @exception  NumberFormatException - If the value of the copy tag was not a valid int.
	* @return the copy number of the document created by a copy operation.
	* The copy number of a new document is 1, every copy of that document gets a 
	* different Copy Number (incremented by 1) in the meta-data.
	*/
	static int getCopyNumber(char* XML);

	/**
	* @exception  NullPointerException - If the XML did not contain the count tag
	* @exception  NumberFormatException - If the value of the count tag was not a valid int.
	* @return the number result of a count operation.
	*/
	static int getCount(char* XML);

	/**
	* @exception  NullPointerException - If the XML did not contain the delete tag
	* @exception  NumberFormatException - If the value of the delete tag was not a valid int.
	* @return the number of deletions made by a delete operation.
	*/
	static int getDeletions(char* XML);

	/**
	* @exception  NullPointerException - If the XML did not contain the insert tag
	* @exception  NumberFormatException - If the value of the insert tag was not a valid int.
	* @return the number of insertions made by a insert operation.
	*/
	static int getInserts(char* XML);

	/**
	* @exception  NullPointerException - If the XML did not contain the modify tag
	* @exception  NumberFormatException - If the value of the modify tag was not a valid int.
	* @return the number of modifications made by a modify operation.
	*/
	static int getModifications(char* XML);

	/**
	* @exception  NullPointerException - If the XML did not contain the modify tag
	* @exception  NumberFormatException - If the value of the modify tag was not a valid int.
	* @return the number of nodes left untouched by a modify operation because they were already set
	* to the modification value. For example, if a document contains two fragments <PRICE>10.90</PRICE>
	* and <PRICE>9.90</PRICE> and a modify command is issued to change all <PRICE> fragments to 9.90, then
	* the second fragment will be left unchanged and the value returned by this method will be 1.
	*/
	static int getModificationMatches(char * XML);


	/**
	* @exception  NullPointerException - If the XML did not contain the store tag
	* @exception  NumberFormatException - If the value of the store tag was not a valid int.
	* @return the number of documents stores by a store operation.
	*/
	static int getStores(char* XML);

	/**
	* @exception  NullPointerException - If the XML did not contain the store tag
	* @exception  NumberFormatException - If the value of the store tag was not a valid int.
	* @return the document ID of the last document stored in a store operation.
	*/
	static int getStoreLastDocID(char* XML);

	/**
	* This checks for all of the error codes.
	*/
	static char* XMLUtils::checkErrors(char* XML);


private:

	static char* getLastTagValue(char* tag, char* XML);
	static int getLastTagIntValue(char* tag, char* XML);
	// Determines whether or not the XML is an Error message created by ErrorCreator
	// on the server. Returns true if it is, false otherwise.
	inline static bool resultIsError(char* XML);
};

#endif XMLUTILS_H
